@@ -3,7 +3,7 @@
  *
  * FileTea, low-friction file sharing <http://filetea.net>
  *
- * Copyright (C) 2011, Igalia S.L.
+ * Copyright (C) 2011-2013, Igalia S.L.
  *
  * Authors:
  *   Eduardo Lima Mitev <elima@igalia.com>
@@ -20,6 +20,8 @@
  * for more details.
  */
 
+#include <uuid/uuid.h>
+
 #include "filetea-node.h"
 #include "file-source.h"
 #include "file-transfer.h"
@@ -37,8 +39,7 @@ G_DEFINE_TYPE (FileteaNode, filetea_node, EVD_TYPE_WEB_SERVICE)
 #define FILETEA_ERROR_DOMAIN_STR "me.filetea.ErrorDomain"
 #define FILETEA_ERROR            g_quark_from_string (FILETEA_ERROR_DOMAIN_STR)
 
-#define DEFAULT_JQUERY_DIR    "/usr/share/javascript/jquery/"
-#define DEFAULT_JQUERY_UI_DIR "/usr/share/javascript/jquery-ui/"
+#define DEFAULT_JQUERY_DIR "/usr/share/javascript/jquery/"
 
 typedef enum {
   FILETEA_ERROR_SUCCESS,
@@ -58,7 +59,6 @@ struct _FileteaNodePrivate
   EvdWebSelector *selector;
   EvdWebDir *webdir;
   EvdWebDir *jquery_webdir;
-  EvdWebDir *jquery_ui_webdir;
 
   GHashTable *sources_by_id;
   GHashTable *sources_by_peer;
@@ -144,29 +144,19 @@ filetea_node_init (FileteaNode *self)
   priv->jquery_webdir = evd_web_dir_new ();
   evd_web_dir_set_alias (priv->jquery_webdir, "/jquery");
 
-  /* jquery-ui web dir */
-  priv->jquery_ui_webdir = evd_web_dir_new ();
-  evd_web_dir_set_alias (priv->jquery_ui_webdir, "/jquery-ui");
-
   /* web selector */
   priv->selector = evd_web_selector_new ();
 
-  evd_web_transport_server_set_selector (priv->transport, priv->selector);
+  evd_web_transport_server_use_selector (priv->transport, priv->selector);
   evd_web_selector_set_default_service (priv->selector,
                                         EVD_SERVICE (priv->webdir));
 
   evd_web_selector_add_service (priv->selector,
                                 NULL,
-                                "^/jquery/",
+                                "/jquery/",
                                 EVD_SERVICE (priv->jquery_webdir),
                                 NULL);
 
-  evd_web_selector_add_service (priv->selector,
-                                NULL,
-                                "^/jquery-ui/",
-                                EVD_SERVICE (priv->jquery_ui_webdir),
-                                NULL);
-
   /* track peers */
   priv->peer_manager = evd_peer_manager_get_default ();
   g_signal_connect (priv->peer_manager,
@@ -268,7 +258,6 @@ filetea_node_finalize (GObject *obj)
   g_object_unref (self->priv->transport);
   g_object_unref (self->priv->webdir);
   g_object_unref (self->priv->jquery_webdir);
-  g_object_unref (self->priv->jquery_ui_webdir);
   g_object_unref (self->priv->selector);
 
   if (self->priv->log_queue != NULL)
@@ -291,10 +280,31 @@ filetea_node_finalize (GObject *obj)
   G_OBJECT_CLASS (filetea_node_parent_class)->finalize (obj);
 }
 
+static gchar *
+generate_random_source_id (const gchar *prefix, gsize len)
+{
+  gchar *id;
+  uuid_t uuid;
+  gchar *id_rnd;
+  gint i;
+
+  uuid_generate (uuid);
+  id_rnd = g_base64_encode (uuid, len);
+  id = g_strconcat (prefix, id_rnd, NULL);
+  g_free (id_rnd);
+
+  for (i=0; id[i] != '\0'; i++)
+    if (id[i] == '/' || id[i] == '+')
+      id[i] = 'x';
+    else if (id[i] == '=')
+      id[i] = '\0';
+
+  return id;
+}
+
 static gchar *
 generate_source_id (FileteaNode *self, const gchar *instance_id)
 {
-  gchar *uuid;
   gchar *id;
   static gint depth;
   gint _depth;
@@ -306,13 +316,10 @@ generate_source_id (FileteaNode *self, const gchar *instance_id)
 
   fails = 0;
 
-  uuid = evd_uuid_new ();
-  uuid[_depth] = '\0';
-  id = g_strconcat (instance_id, uuid, NULL);
+  id = generate_random_source_id (instance_id, _depth);
 
   while (g_hash_table_lookup (self->priv->sources_by_id, id) != NULL)
     {
-      g_free (uuid);
       g_free (id);
 
       fails++;
@@ -323,13 +330,9 @@ generate_source_id (FileteaNode *self, const gchar *instance_id)
           fails = 0;
         }
 
-      uuid = evd_uuid_new ();
-      uuid[_depth] = '\0';
-      id = g_strconcat (instance_id, uuid, NULL);
+      id = generate_random_source_id (instance_id, _depth);
     }
 
-  g_free (uuid);
-
   return id;
 }
 
@@ -762,6 +765,49 @@ user_agent_is_browser (const gchar *user_agent)
     (g_strstr_len (user_agent, strlen (opera), opera) == user_agent);
 }
 
+static void
+check_file_size_changed (FileteaNode    *self,
+                         FileTransfer   *transfer,
+                         EvdHttpRequest *request)
+{
+  SoupMessageHeaders *headers;
+
+  headers = evd_http_message_get_headers (EVD_HTTP_MESSAGE (request));
+  if (soup_message_headers_get_encoding (headers) ==
+      SOUP_ENCODING_CONTENT_LENGTH)
+    {
+      gsize reported_content_len;
+
+      reported_content_len = soup_message_headers_get_content_length (headers);
+      if (reported_content_len != transfer->source->file_size)
+        {
+          JsonNode *node;
+          JsonArray *args;
+
+          transfer->source->file_size = reported_content_len;
+
+          node = json_node_new (JSON_NODE_ARRAY);
+          args = json_array_new ();
+          json_node_set_array (node, args);
+
+          json_array_add_string_element (args, transfer->source->id);
+          json_array_add_int_element (args, transfer->source->file_size);
+
+          if (! evd_jsonrpc_send_notification (self->priv->rpc,
+                                               "update-file-size",
+                                               node,
+                                               transfer->source->peer,
+                                               NULL))
+            {
+              g_warning ("Failed to send 'transfer-started' notification to peer");
+            }
+
+          json_array_unref (args);
+          json_node_free (node);
+        }
+    }
+}
+
 static gboolean
 handle_special_request (FileteaNode         *self,
                         EvdHttpConnection   *conn,
@@ -785,6 +831,9 @@ handle_special_request (FileteaNode         *self,
           JsonNode *node;
           JsonArray *args;
 
+          /* check if file size has changed */
+          check_file_size_changed (self, transfer, request);
+
           file_transfer_set_source_conn (transfer, conn);
           file_transfer_start (transfer);
 
@@ -1232,7 +1281,7 @@ setup_web_dir_logging (FileteaNode *self)
 static void
 load_config (FileteaNode *self, GKeyFile *config)
 {
-  gchar *dir;
+  gchar *jquery_dir;
 
   /* source id start depth */
   self->priv->source_id_start_depth =
@@ -1244,7 +1293,7 @@ load_config (FileteaNode *self, GKeyFile *config)
     self->priv->source_id_start_depth = DEFAULT_SOURCE_ID_START_DEPTH;
   else
     self->priv->source_id_start_depth =
-      MAX (self->priv->source_id_start_depth, strlen (self->priv->id) + 1);
+      MIN (self->priv->source_id_start_depth, 16 + strlen (self->priv->id));
 
   /* global bandwidth limites */
   set_max_node_bandwidth (self,
@@ -1279,26 +1328,15 @@ load_config (FileteaNode *self, GKeyFile *config)
     setup_web_dir_logging (self);
 
   /* jquery dir */
-  dir = g_key_file_get_string (config,
-                               "node",
-                               "jquery-dir",
-                               NULL);
-  if (dir == NULL || dir[0] == '\0')
-    dir = g_strdup (DEFAULT_JQUERY_DIR);
+  jquery_dir = g_key_file_get_string (config,
+                                      "node",
+                                      "jquery-dir",
+                                      NULL);
+  if (jquery_dir == NULL || jquery_dir[0] == '\0')
+    jquery_dir = g_strdup (DEFAULT_JQUERY_DIR);
 
-  evd_web_dir_set_root (self->priv->jquery_webdir, dir);
-  g_free (dir);
-
-  /* jquery-ui dir */
-  dir = g_key_file_get_string (config,
-                               "node",
-                               "jquery-ui-dir",
-                               NULL);
-  if (dir == NULL || dir[0] == '\0')
-    dir = g_strdup (DEFAULT_JQUERY_UI_DIR);
-
-  evd_web_dir_set_root (self->priv->jquery_ui_webdir, dir);
-  g_free (dir);
+  evd_web_dir_set_root (self->priv->jquery_webdir, jquery_dir);
+  g_free (jquery_dir);
 }
 
 /* public methods */
